-------------------------------------------------------------------
-- Date 		: 06 Feb 2011
-- Databse		: DB2 9.7
-- Description 		: ISAMESSO Cached Wallet Maintenance Script
-- IMS Version		: 8.1.x / 8.2.x
--
--	Syntax 		: call sp_pruneCachedWallet('<del mode>','<retain date>',<max user>);
--  * <del mode> - Mandatory, text, case sensitive  
--				 - ''ABOUT'' displays script information 
--				 - ''DELETE'' performs actual delete 
--				 - ''HELP'' displays script syntax and example 
--				 - ''SIMULATE'' simulates deletion without actual delete 
--	* <retain date>	- Optional, text, date in YYYYMMDD format
--					- Process only Cached Wallets created before specified retain date
--					- Leave blank to process all Cached Wallets
--	* <max users>  - Mandatory, numeric
--				   - Maximum number of ISAMESSO users to be processed
--				   - Enter 0 to process all the ISAMESSO users
-------------------------------------------------------------------
--
-- call sp_pruneCachedWallet('SIMULATE','',5000)- Simulate deletion of up to 5K random users' dangling cached wallets
-- call sp_pruneCachedWallet('DELETE','20110101',10000)	- Delete up to 10K random users' dangling cached wallets created before 01-Jan-2011
-- call sp_pruneCachedWallet('SIMULATE','',0) - Simulate deletion of all users' dangling cached wallets without performing actual delete. 
--
-- DROP PROCEDURE sp_pruneCachedWallet
-- call sp_pruneCachedWallet('SIMULATE','',0)
-- call sp_pruneCachedWallet('SIMULATE','',50)
-- call sp_pruneCachedWallet('DELETE','',50)
-- call sp_pruneCachedWallet('ABOUT','',0)
-- call sp_pruneCachedWallet('HELP','',0)
--
-- Tables affected : 
--  IMSCertificate 
--  IMSSociUniqueAttribute 
--	IMSSociAttribute
--	IMSSoci
--
-- Changes Log :
--
-- v1.0.2 - v9.5 compatible fix - Cast all integer value with TRIM() and CHAR() for String concatenation 
--        - v9.5 compatible fix - Cast DATE() and TIME() with CHAR() for String concatenation 
--        - v9.5 compatible fix - Replace "ROUND()" with "INTEGER()" 
--	 	  - Added IMS schema check
-- 		  - Change the load of ISAMESSO user working list from IMSIdentity to IMSSoci
-- 		  - Skip randomise ISAMESSO user list process if limit is NOT specified
--        - Added 'ON COMMIT PRESERVE ROWS' for temporary table creation
--		  - Skip temporary table processing for 'SIMULATE' mode
--		  - Combine 2 steps (INSERT & UPDATE) of temporary processing into 1 (INSERT)
--		  - Interface change from walletDeleteLimit to userProcessLimit to allow better 
-- 			estimation of schedule job time 
--
-- v1.0.1 - Added "COMMIT;" after "BEGIN ATOMIC ...<Delete cached wallet data>.. END"
--        - Remove unnecessary SPECIFIC from stored procedure declaration  

CREATE PROCEDURE sp_pruneCachedWallet
(
IN IN_operationCmd varchar(16) 
,IN IN_retainDate_yyyymmdd varchar(8)
,IN IN_userProcessLimit integer
)
DYNAMIC RESULT SETS 1
MODIFIES SQL DATA
LANGUAGE SQL 
BEGIN
	
	-- script variable
	DECLARE @info_script_ver varchar(8) DEFAULT '1.0.2';
	DECLARE @info_start_timestamp TIMESTAMP;
	DECLARE @info_end_timestamp TIMESTAMP;
	DECLARE @info_elapsed_time varchar(32);
	
	DECLARE @info_total_wallet_processed INTEGER DEFAULT 0;
	DECLARE @info_total_wallet_deleted INTEGER DEFAULT 0;
	DECLARE @wallet_per_batch INTEGER DEFAULT 0;
	DECLARE @total_wallet_pending_del INTEGER DEFAULT 0;
	DECLARE @wallet_del_per_batch INTEGER DEFAULT 0;
	DECLARE @wallet_prune_before_yyyymmdd INTEGER;
	
	DECLARE @total_unprocessed_imsid INTEGER DEFAULT 0;	
	DECLARE @info_total_imsid_processed INTEGER DEFAULT 0;
	
	DECLARE @imsid_start_rowid INTEGER DEFAULT 0;	

	DECLARE @val_message varchar(1024);
	DECLARE @exception_msg varchar(128) DEFAULT NULL;
	DECLARE @val_temp_int INTEGER;
	DECLARE @var_del_tmpCnt INTEGER;
	DECLARE @val_temp_time TIMESTAMP;
	DECLARE @sql_str varchar(1024) DEFAULT NULL;
	
	
	-- Audit log, 1:enabled 0:disabled
	DECLARE @debug_log_enabled INTEGER DEFAULT 0;	
	DECLARE @debug_logid VARCHAR(32);
	DECLARE @debug_logseq INTEGER DEFAULT 0;
	DECLARE @debug_log_event VARCHAR(512);
	
	--Performance Tuning Variable
	DECLARE @imsid_per_batch INTEGER DEFAULT 200;
	DECLARE @max_del_per_batch INTEGER DEFAULT 10000;
	--Exclude wallet from deletion in specified days
	DECLARE @wallet_retain_days INTEGER DEFAULT 30;
	--Cached wallet delete limit, 0 = no limit
	DECLARE @wallet_delete_limit INTEGER DEFAULT 0;
	--IMS max user, 0
	DECLARE @user_process_limit INTEGER DEFAULT 0;
	-- Randomise user list before processing, 1:enabled 0:disabled
	DECLARE @is_randomise_user_list INTEGER DEFAULT 1;	
	
	-- result output
	DECLARE @cursor_result_output CURSOR WITH RETURN TO CLIENT FOR
	SELECT msg from
	(
		SELECT 0 id, ('command : sp_pruneCachedWallet ('''|| IN_operationCmd || ''',''' || IN_retainDate_yyyymmdd || ''',' || TRIM(CHAR(@user_process_limit)) || ')' ) msg FROM SYSIBM.SYSDUMMY1
		union SELECT 11 id, ('  Input  > Delete mode : ' || IN_operationCmd) msg FROM SYSIBM.SYSDUMMY1
		union SELECT 12 id, ('  Input  > Retain date : ' || CASE WHEN IN_retainDate_yyyymmdd = '' THEN 'not specified' ELSE IN_retainDate_yyyymmdd END) msg FROM SYSIBM.SYSDUMMY1
		union SELECT 13 id, ('  Input  > Maximum no. of users : ' || CASE WHEN @user_process_limit = 0 THEN 'no limit' ELSE TRIM(CHAR(@user_process_limit)) END) msg FROM SYSIBM.SYSDUMMY1
		--union SELECT 14 id, ('  Input  > Wallet delete limit : ' || CASE WHEN @wallet_delete_limit = 0 THEN 'no limit' ELSE TRIM(CHAR(@wallet_delete_limit)) END) msg FROM SYSIBM.SYSDUMMY1

		union SELECT 21 id, ('  Output > Start time : ' || CHAR(date(@info_start_timestamp)) || ' ' || CHAR(time(@info_start_timestamp)) ) msg FROM SYSIBM.SYSDUMMY1
		union SELECT 22 id, ('  Output > End time : ' || CHAR(date(@info_end_timestamp)) || ' ' || CHAR(time(@info_end_timestamp)) ) msg FROM SYSIBM.SYSDUMMY1
		union SELECT 23 id, ('  Output > Total elapsed time : ' || @info_elapsed_time) msg FROM SYSIBM.SYSDUMMY1
		
		union SELECT 31 id, ('  Output > Total ISAMESSO user(s) processed : ' || TRIM(CHAR(@info_total_imsid_processed)) ) msg FROM SYSIBM.SYSDUMMY1
		union SELECT 32 id, ('  Output > Total Cached Wallet(s) processed : ' || TRIM(CHAR(@info_total_wallet_processed)) ) msg FROM SYSIBM.SYSDUMMY1
		union SELECT 33 id, ('  Output > Total Cached Wallet(s) ' || CASE WHEN IN_operationCmd = 'DELETE' THEN 'deleted' ELSE 'pending deletion' END || ' : ' || TRIM(CHAR(@info_total_wallet_deleted)) ) msg FROM SYSIBM.SYSDUMMY1
		
		union SELECT 50 id, (CASE WHEN @debug_log_enabled = 1 THEN '  Debug > Log : SELECT LOGID,LOGSEQ,LOGTIMESTART,LOGTIMEEND,(LOGTIMEEND-LOGTIMESTART)ELAPSED,LOGEVENT FROM SESSION.tmpTblCachedWalletLog WHERE LogID = ''' || TRIM(CHAR(@debug_logid)) || '''' END) msg FROM SYSIBM.SYSDUMMY1
		--union SELECT 50 id, (CASE WHEN @debug_log_enabled = 1 THEN 
		--	(SELECT LOGID||'-'||TRIM(CHAR(LOGSEQ))||'-'||CHAR(LOGTIMESTART)||'-'||CHAR(LOGTIMEEND)||'-'||TRIM(CHAR((LOGTIMEEND-LOGTIMESTART)))||'-'||LOGEVENT FROM SESSION.tmpTblCachedWalletLog WHERE LogID = @debug_logid)  
		--	 END) msg FROM SYSIBM.SYSDUMMY1
		
		
	) order by id;

	-- Exception output
	DECLARE @cursor_syntax CURSOR WITH RETURN TO CLIENT FOR
	SELECT msg from
	(
		SELECT 0 id, ('command : sp_pruneCachedWallet ('''|| IN_operationCmd || ''',''' || IN_retainDate_yyyymmdd || ''',' || TRIM(CHAR(@wallet_delete_limit)) || ')' ) msg FROM SYSIBM.SYSDUMMY1
		union SELECT 11 id,  CASE WHEN @val_message IS NULL OR TRIM(@val_message) = '' THEN '' ELSE ' * ' || @val_message END msg FROM SYSIBM.SYSDUMMY1
		union SELECT 12 id, ('Syntax : sp_pruneCachedWallet( ''<del mode>'', ''<retain date>'', <max users>) ' )msg FROM SYSIBM.SYSDUMMY1
		union SELECT 13 id, (' * <del mode> - Mandatory, text, case sensitive ' )msg FROM SYSIBM.SYSDUMMY1
		union SELECT 14 id, (' * 			- ''ABOUT'' displays script information' )msg FROM SYSIBM.SYSDUMMY1
		union SELECT 15 id, (' * 			- ''DELETE'' to perform actual delete' )msg FROM SYSIBM.SYSDUMMY1
		union SELECT 16 id, (' * 			- ''HELP'' displays script syntax and examples' )msg FROM SYSIBM.SYSDUMMY1
		union SELECT 17 id, (' * 			- ''SIMULATE'' to simulate deletion without actual delete' )msg FROM SYSIBM.SYSDUMMY1
			   
		union SELECT 21 id, (' * <retain date> 	- Optional, text, date in YYYYMMDD format' )msg FROM SYSIBM.SYSDUMMY1
		union SELECT 22 id, (' * 				- Process only Cached Wallets created before specified retain date' )msg FROM SYSIBM.SYSDUMMY1
		union SELECT 23 id, (' * 				- Leave blank to process all Cached Wallets' )msg FROM SYSIBM.SYSDUMMY1
		
		union SELECT 41 id, (' * <max users> - Optional, numeric' )msg FROM SYSIBM.SYSDUMMY1
		union SELECT 42 id, (' * 			- Maximum number of ISAMESSO users to be processed' )msg FROM SYSIBM.SYSDUMMY1
		union SELECT 43 id, (' * 			- Enter 0 to process all the ISAMESSO users' )msg FROM SYSIBM.SYSDUMMY1

		union SELECT 51 id, ('Example : ' )msg FROM SYSIBM.SYSDUMMY1
		union SELECT 52 id, (' * sp_pruneCachedWallet(''SIMULATE'','''',5000) - Simulate deletion up to 5K random users'' dangling cached wallet' )msg FROM SYSIBM.SYSDUMMY1
		union SELECT 53 id, (' * sp_pruneCachedWallet(''DELETE'',''20110101'',10000) - Delete up to 10K random users'' dangling cached wallet created before 01-Jan-2011' )msg FROM SYSIBM.SYSDUMMY1
		union SELECT 54 id, (' * sp_pruneCachedWallet(''SIMULATE'','''',0) - Simulate deletion of all users'' dangling cached wallet without performing actual delete' )msg FROM SYSIBM.SYSDUMMY1
		
		
	) order by id;
	

	DECLARE @cursor_about CURSOR WITH RETURN TO CLIENT FOR
	SELECT msg from
	(
		SELECT  1 id, (' ISAMESSO Cached Wallet Maintenance Script' )msg FROM SYSIBM.SYSDUMMY1
		union SELECT  2 id, ('  - Database Name    : ' || CURRENT SERVER )msg FROM SYSIBM.SYSDUMMY1
		union SELECT  3 id, ('  - Schema Name      : ' || CURRENT SCHEMA )msg FROM SYSIBM.SYSDUMMY1
		union SELECT  4 id, ('  - Script Version   : ' || @info_script_ver )msg FROM SYSIBM.SYSDUMMY1
		union SELECT  5 id, ('  - Database Version : ' || (SELECT service_level FROM TABLE (sysproc.env_get_inst_info())) )msg FROM SYSIBM.SYSDUMMY1
	) order by id;


	-- Handle SQLException
	DECLARE EXIT HANDLER FOR SQLEXCEPTION
	BEGIN
		GET DIAGNOSTICS EXCEPTION 1 @exception_msg = MESSAGE_TEXT;
		
		IF @val_message IS NULL THEN
			SET @val_message = @exception_msg;
		END IF;
		
		IF IN_operationCmd = 'ABOUT' THEN
			IF TRIM(@val_message) = '' THEN
				OPEN @cursor_about;
			ELSE
				OPEN @cursor_syntax;
			END IF;
		ELSEIF IN_operationCmd = 'HELP' THEN
			OPEN @cursor_syntax;
		ELSE
			OPEN @cursor_syntax;
		END IF;
	END;
	
	IF (@debug_log_enabled = 1) THEN
		DECLARE GLOBAL TEMPORARY TABLE SESSION.tmpTblCachedWalletLog
		(
			LogID VARCHAR(32) NOT NULL,
			LogSeq INTEGER,
			LogTimeStart TIMESTAMP NOT NULL,
			LogTimeEnd TIMESTAMP,
			LogEvent VARCHAR (512) NOT NULL
	  	) WITH REPLACE NOT LOGGED ON COMMIT PRESERVE ROWS;
	 	CREATE UNIQUE INDEX SESSION.IDX_TMP_LOGID ON SESSION.tmpTblCachedWalletLog (LogID, LogSeq);
	END IF;
	
	DECLARE GLOBAL TEMPORARY TABLE SESSION.tmpTblCachedWalletImsID
	(
		ROWID INTEGER NOT NULL,
		IMSID VARCHAR (128) NOT NULL
  	) WITH REPLACE NOT LOGGED ON COMMIT PRESERVE ROWS;
	CREATE UNIQUE INDEX SESSION.IDX_TMP_IMSID ON SESSION.tmpTblCachedWalletImsID (ROWID);
  	
  	DECLARE GLOBAL TEMPORARY TABLE SESSION.tmpTblCachedWalletDelete
	(
		ROWID INTEGER NOT NULL,
		IMSID VARCHAR (128) NOT NULL,
		CREATIONHOST VARCHAR (128) NOT NULL,
		CREATIONYYYYMMDD INTEGER NOT NULL DEFAULT 0,
		CREATIONDAYS INTEGER NOT NULL DEFAULT 0,
		sociID VARCHAR (128) NOT NULL
		--,DELETESEQ INTEGER DEFAULT 0
	) WITH REPLACE NOT LOGGED ON COMMIT PRESERVE ROWS;
	CREATE UNIQUE INDEX SESSION.IDX_TMP_SOCI ON SESSION.tmpTblCachedWalletDelete (sociID);
	CREATE UNIQUE INDEX SESSION.IDX_TMP_SOCIROWID ON SESSION.tmpTblCachedWalletDelete (ROWID);
	
	-- Check is IMS schema
	SELECT DISTINCT COUNT(TABNAME) INTO @val_temp_int FROM SYSCAT.TABLES 
		WHERE UPPER(TABNAME) LIKE 'IMSSOCI%' AND TABSCHEMA = CURRENT SCHEMA;
	
	-- Valid IMS schema will have at least 3 IMSSociXXX tables
	IF @val_temp_int <3 THEN
		SET @val_message = 'Error : Invalid ISAMESSO schema - ' || CURRENT SCHEMA;
		SELECT INTEGER('A') INTO @val_temp_int FROM SYSIBM.SYSDUMMY1;
	END IF;
	
	
	SET @user_process_limit = IN_userProcessLimit;
	--SET @wallet_delete_limit = IN_walletDeleteLimit;
	
	-- ## INPUT VALIDATION
	IF IN_operationCmd = 'ABOUT' THEN
		-- force SQLException to exit with ABOUT info
		SET @val_message = '';
		SELECT INTEGER('A') INTO @val_temp_int FROM SYSIBM.SYSDUMMY1;
	ELSEIF IN_operationCmd = 'HELP' THEN
		SET @val_message = '';
		-- force SQLException to exit with HELP
		SELECT INTEGER('A') INTO @val_temp_int FROM SYSIBM.SYSDUMMY1;
		
	ELSEIF IN_operationCmd = 'SIMULATE' THEN
	 	-- SIMULATE
		-- do nothing
	ELSEIF IN_operationCmd = 'DELETE' THEN
		-- DELETE
		-- do nothing
	ELSE
		-- syntax error , force SQLException to exit with HELP
		SET @val_message = 'Error : Invalid delete mode.';
		SELECT INTEGER('A') INTO @val_temp_int FROM SYSIBM.SYSDUMMY1;
	END IF;			
			
			
	IF IN_retainDate_yyyymmdd = '' THEN
		-- Set it to year 3000 (no retain)  
		SET @wallet_prune_before_yyyymmdd = 30000000;
	ELSE
		-- validate date format
		SET @val_message = 'Error : Invalid retain date.';
		SELECT TO_DATE( IN_retainDate_yyyymmdd, 'YYYYMMDD') INTO @val_temp_time FROM SYSIBM.SYSDUMMY1;
		SET @val_message = NULL;
		
		-- if above SQL execute without SQLException get the integer value of yyyyMMdd
		SET @wallet_prune_before_yyyymmdd = INTEGER(IN_retainDate_yyyymmdd);		
	END IF;
	
	-- if a delete limit is specified
	IF @wallet_delete_limit > 0 THEN
		SET @total_wallet_pending_del = @wallet_delete_limit;
		
	ELSEIF @wallet_delete_limit < 0 THEN
		SET @val_message = 'Error : Invalid Cached Wallet delete limit.';
		SELECT INTEGER('A') INTO @val_temp_int FROM SYSIBM.SYSDUMMY1;
	END IF;
	
	IF @user_process_limit < 0 THEN
		SET @val_message = 'Error : Invalid maximum number of user.';
		SELECT INTEGER('A') INTO @val_temp_int FROM SYSIBM.SYSDUMMY1;
	END IF;	
	-- ## END OF INPUT VALIDATION
	
	--Generate logID based on timestamp 
	SELECT CHAR(CURRENT TIMESTAMP) INTO @debug_logid FROM SYSIBM.SYSDUMMY1;	
	
	-- ### Audit Log ###--
	IF (@debug_log_enabled = 1) THEN
		SET @debug_log_event = '[SCRIPT_START]';
		SET @debug_logseq = @debug_logseq +1;
		INSERT INTO SESSION.tmpTblCachedWalletLog (LogID, LogSeq, LogTimeStart, LogEvent) VALUES (@debug_logid, @debug_logseq, CURRENT TIMESTAMP, @debug_log_event);
	END IF;
	
	--capture start timestamp
	SELECT CURRENT TIMESTAMP INTO @info_start_timestamp FROM SYSIBM.SYSDUMMY1;

	-- clear & populate the IMS ID temp table	
	DELETE FROM SESSION.tmpTblCachedWalletImsID;	

	-- ### Audit Log ###--
	IF (@debug_log_enabled = 1) THEN
		SET @debug_log_event = '--INSERT_IMSID';
		SET @debug_logseq = @debug_logseq +1;
		INSERT INTO SESSION.tmpTblCachedWalletLog (LogID, LogSeq, LogTimeStart, LogEvent) VALUES (@debug_logid, @debug_logseq, CURRENT TIMESTAMP, @debug_log_event);
	END IF;
	
	IF (@user_process_limit > 0) THEN
		SET @sql_str = 'INSERT INTO SESSION.tmpTblCachedWalletImsID (ROWID, IMSID) ';
		-- Randomise IMSId user		
		IF (@is_randomise_user_list=1) THEN
			SET @sql_str = @sql_str || ' SELECT ROW_NUMBER() OVER (ORDER BY RandID), imsID FROM (SELECT imsID, RAND() RandID FROM IMSSoci where sociID like ''200%'' GROUP BY imsID FETCH FIRST '||CHAR(@user_process_limit)||' ROWS ONLY	) ';		
			--SET @sql_str = @sql_str || ' SELECT ROW_NUMBER() OVER (ORDER BY RandID), imsID FROM (SELECT RAND() RandID, imsID FROM IMSSoci where sociID like ''200%'' GROUP BY imsID) ';		
		ELSE
			SET @sql_str = @sql_str || ' SELECT imsID,ROW_NUMBER() OVER (ORDER BY IMSID) FROM IMSSoci where sociID like ''200%'' GROUP BY imsID FETCH FIRST '||CHAR(@user_process_limit)||' ROWS ONLY	';		
			--SET @sql_str = @sql_str || ' SELECT ROW_NUMBER() OVER (ORDER BY IMSID), imsID FROM IMSSoci where sociID like ''200%'' GROUP BY imsID ';
		END IF;
		--SET @sql_str = @sql_str || ' FETCH FIRST '||CHAR(@user_process_limit)||' ROWS ONLY	';			
		EXECUTE IMMEDIATE @sql_str;
	ELSE
		INSERT INTO SESSION.tmpTblCachedWalletImsID (ROWID, IMSID) 
			SELECT ROW_NUMBER() OVER (ORDER BY IMSID), imsID FROM IMSSoci where sociID like '200%' GROUP BY imsID;
	END IF;
		
	-- ### Audit Log ###--
	IF (@debug_log_enabled = 1) THEN
		SET @debug_log_event = '--INSERT_IMSID';
		Update SESSION.tmpTblCachedWalletLog SET LogTimeEnd = CURRENT TIMESTAMP WHERE LogID = @debug_logid AND LogEvent = @debug_log_event;
	END IF;

	-- Get the total number of unprocessed IMSID 
	SELECT 
		COUNT(IMSID) INTO @total_unprocessed_imsid
	FROM SESSION.tmpTblCachedWalletImsID;
	
	IF (@user_process_limit > 0)AND(@total_unprocessed_imsid > @user_process_limit) THEN
		SET @total_unprocessed_imsid = @user_process_limit;
	END IF;
	
	
	-- ###[ WHILE SESSION.tmpTblCachedWalletImsID IS NOT EMPTY) ]###--
	WHILE (@total_unprocessed_imsid > 0) DO

		-- Get current batch start imsID start rowid
		SELECT MIN(ROWID) INTO @imsid_start_rowid FROM SESSION.tmpTblCachedWalletImsID;
	
		-- check if batch number is greater than outstanding imsid
		IF @imsid_per_batch > @total_unprocessed_imsid THEN
			SET @imsid_per_batch = @total_unprocessed_imsid;
		END IF;
	
		-- calc total wallet processed
		SELECT COUNT(sociID) INTO @wallet_per_batch FROM IMSSoci 
		WHERE imsID IN
		   (
		   SELECT IMSID FROM SESSION.tmpTblCachedWalletImsID WHERE ROWID < (@imsid_start_rowid + @imsid_per_batch)
		   )
		   AND sociID like '200%';			
	  		
		-- ### Audit Log ###--
		IF (@debug_log_enabled = 1) THEN
			SET @debug_log_event = '--IMSID_BATCH '||TRIM(CHAR(@imsid_start_rowid))||'-'||TRIM(CHAR(@imsid_start_rowid+@imsid_per_batch-1))||' total wallet : '||TRIM(CHAR(@wallet_per_batch)) ||' ...';
			SET @debug_logseq = @debug_logseq +1;		
			INSERT INTO SESSION.tmpTblCachedWalletLog (LogID, LogSeq, LogTimeStart, LogEvent) VALUES (@debug_logid, @debug_logseq, CURRENT TIMESTAMP, @debug_log_event);
		END IF;
		
		IF IN_operationCmd = 'DELETE' THEN
			
			-- Remove previous batch cached wallet data
			DELETE FROM SESSION.tmpTblCachedWalletDelete;
			
			IF (@debug_log_enabled = 1) THEN
				SET @debug_log_event = '----DELMODE_INSERT_WALLET';
				SET @debug_logseq = @debug_logseq +1;		
				INSERT INTO SESSION.tmpTblCachedWalletLog (LogID, LogSeq, LogTimeStart, LogEvent) VALUES (@debug_logid, @debug_logseq, CURRENT TIMESTAMP, @debug_log_event);
			END IF;
			
			-- Retrieve current batch of ISAMESSO user's cached wallet & calc age of wallet
			INSERT INTO SESSION.tmpTblCachedWalletDelete (ROWID, IMSID, sociID, CREATIONHOST, CREATIONYYYYMMDD, CREATIONDAYS )
			SELECT 
				ROW_NUMBER() OVER (ORDER BY IMSID),
				IMSID, SOCIID, CREATIONHOST, CREATIONYYYYMMDD, CREATIONDAYS
			FROM
			(
				SELECT 
					S.IMSID
					,COALESCE(SA1.ATTRVALUE,'-') CREATIONHOST			
					,COALESCE(SA2.ATTRVALUE,'0') CREATIONTIME
					,INTEGER(SUBSTR(SA2.ATTRVALUE,1,8)) CREATIONYYYYMMDD
					,DAYS(CURRENT DATE) - DAYS(TO_DATE(SUBSTR(SA2.ATTRVALUE,1,8), 'YYYYMMDD') ) CREATIONDAYS  	
					,S.SOCIID
				FROM 
				  ( 
				  	--SELECT DISTINCT IMSID FROM IMSSoci WHERE sociID like '200%' FETCH FIRST 200 ROWS ONLY
				  	SELECT IMSID FROM SESSION.tmpTblCachedWalletImsID WHERE ROWID < (@imsid_start_rowid + @imsid_per_batch) 
				  ) T
			   	  INNER JOIN IMSSoci S ON (S.IMSID = T.IMSID AND S.sociID like '200%') 
				  INNER JOIN IMSSociAttribute SA1 ON (SA1.sociID = S.sociID AND SA1.ATTRNAME = 'keyCreationHost') 
				  INNER JOIN IMSSociAttribute SA2 ON (SA2.sociID = S.sociID AND SA2.ATTRNAME = 'keyCreationTime')
			)	
			WHERE 1=1 
			AND (IMSID, CREATIONHOST, CREATIONTIME) NOT IN
		  	(
				SELECT 
					S.IMSID
					,COALESCE(SA1.ATTRVALUE,'-') CREATIONHOST			
					,MAX(COALESCE(SA2.ATTRVALUE,'0')) CREATIONTIME  			
				FROM 
				  ( 
				  	--SELECT DISTINCT IMSID FROM IMSSoci WHERE sociID like '200%' FETCH FIRST 200 ROWS ONLY
				  	SELECT IMSID FROM SESSION.tmpTblCachedWalletIMSID WHERE ROWID < (@imsid_start_rowid + @imsid_per_batch)
				  ) T
			   	  INNER JOIN IMSSoci S ON (S.IMSID = T.IMSID AND S.sociID like '200%') 
				  INNER JOIN IMSSociAttribute SA1 ON (SA1.sociID = S.sociID AND SA1.ATTRNAME = 'keyCreationHost') 
				  INNER JOIN IMSSociAttribute SA2 ON (SA2.sociID = S.sociID AND SA2.ATTRNAME = 'keyCreationTime')
			  	GROUP BY
			  		S.IMSID, 
			  		SA1.ATTRVALUE
		  	)
			AND CREATIONDAYS > @wallet_retain_days
			AND CREATIONYYYYMMDD < @wallet_prune_before_yyyymmdd;			
			
			IF (@debug_log_enabled = 1) THEN
				--SET @debug_log_event = '----DELMODE_INSERT_WALLET';
				Update SESSION.tmpTblCachedWalletLog SET LogTimeEnd = CURRENT TIMESTAMP WHERE LogID = @debug_logid AND LogSeq = @debug_logseq;			
			END IF;

		  	-- Count cached wallet mark for delete 
			SELECT COUNT(sociID) INTO @wallet_del_per_batch
				FROM SESSION.tmpTblCachedWalletDelete;
	
			-- If deletion limit is specified
			IF @total_wallet_pending_del > 0 THEN
			
				-- Recalc the delete limit if more than allowed deletion
				IF @wallet_del_per_batch>= @total_wallet_pending_del THEN
					SET @wallet_del_per_batch= @total_wallet_pending_del;
					SET @total_unprocessed_imsid = 0; -- force exit 		
				END IF;
				
				SET @total_wallet_pending_del = @total_wallet_pending_del - @wallet_del_per_batch;
			END IF;
			
			-- Delete cached wallet data
			IF (@debug_log_enabled = 1) THEN
				SET @debug_log_event = '----DELMODE_DELETE_WALLET';
				SET @debug_logseq = @debug_logseq +1;		
				INSERT INTO SESSION.tmpTblCachedWalletLog (LogID, LogSeq, LogTimeStart, LogEvent) VALUES (@debug_logid, @debug_logseq, CURRENT TIMESTAMP, @debug_log_event);
			END IF;
			
			SET @var_del_tmpCnt = 1;
			WHILE (@var_del_tmpCnt > 0) DO
				BEGIN ATOMIC
					SET @sql_str = ' WHERE sociID IN ( SELECT sociID FROM SESSION.tmpTblCachedWalletDelete WHERE ROWID > 0 AND ROWID <= '||CHAR(@wallet_del_per_batch)||' FETCH FIRST '||CHAR(@max_del_per_batch)||' ROWS ONLY )';

					EXECUTE IMMEDIATE 'DELETE FROM IMSCertificate '||@sql_str;
					EXECUTE IMMEDIATE 'DELETE FROM IMSSociUniqueAttribute '||@sql_str;
					EXECUTE IMMEDIATE 'DELETE FROM IMSSociAttribute '||@sql_str;
					EXECUTE IMMEDIATE 'DELETE FROM IMSSoci '||@sql_str;
					
					GET DIAGNOSTICS @var_del_tmpCnt = ROW_COUNT;
				END;
				COMMIT;
			END WHILE;
			
			IF (@debug_log_enabled = 1) THEN
				--SET @debug_log_event = '----DELMODE_DELETE_WALLET';	
				--Update SESSION.tmpTblCachedWalletLog SET LogTimeEnd = CURRENT TIMESTAMP WHERE LogID = @debug_logid AND LogEvent = @debug_log_event;
				Update SESSION.tmpTblCachedWalletLog SET LogTimeEnd = CURRENT TIMESTAMP WHERE LogID = @debug_logid AND LogSeq = @debug_logseq;
			END IF;			

			
		ELSEIF IN_operationCmd = 'SIMULATE' THEN
		
			IF (@debug_log_enabled = 1) THEN
				SET @debug_log_event = '----SIMULATE_MODE_CALC_WALLET';
				SET @debug_logseq = @debug_logseq +1;		
				INSERT INTO SESSION.tmpTblCachedWalletLog (LogID, LogSeq, LogTimeStart, LogEvent) VALUES (@debug_logid, @debug_logseq, CURRENT TIMESTAMP, @debug_log_event);
			END IF;		
		
			-- Count duplicated cached wallet with >30 days of creation
			SELECT COUNT(*) INTO @wallet_del_per_batch
			FROM
			(
				SELECT 
					S.IMSID
					,COALESCE(SA1.ATTRVALUE,'-') CREATIONHOST			
					,COALESCE(SA2.ATTRVALUE,'0') CREATIONTIME
					,INTEGER(SUBSTR(SA2.ATTRVALUE,1,8)) CREATIONYYYYMMDD
					,DAYS(CURRENT DATE) - DAYS(TO_DATE(SUBSTR(SA2.ATTRVALUE,1,8), 'YYYYMMDD') ) CREATIONDAYS 
					,S.sociID
				FROM 
				  ( 
				  	--SELECT DISTINCT IMSID FROM IMSSoci WHERE sociID like '200%' FETCH FIRST 200 ROWS ONLY
				  	SELECT IMSID FROM SESSION.tmpTblCachedWalletImsID WHERE ROWID < (@imsid_start_rowid + @imsid_per_batch) 
				  ) T
			   	  INNER JOIN IMSSoci S ON (S.IMSID = T.IMSID AND S.sociID like '200%') 
				  INNER JOIN IMSSociAttribute SA1 ON (SA1.sociID = S.sociID AND SA1.ATTRNAME = 'keyCreationHost') 
				  INNER JOIN IMSSociAttribute SA2 ON (SA2.sociID = S.sociID AND SA2.ATTRNAME = 'keyCreationTime')
			)	
			WHERE 1=1 
			AND (IMSID, CREATIONHOST, CREATIONTIME) NOT IN
		  	(
				SELECT 
					S.IMSID
					,COALESCE(SA1.ATTRVALUE,'-') CREATIONHOST			
					,MAX(COALESCE(SA2.ATTRVALUE,'0')) CREATIONTIME 
				FROM 
				  ( 
				  	--SELECT DISTINCT IMSID FROM IMSSoci WHERE sociID like '200%' FETCH FIRST 200 ROWS ONLY
				  	SELECT IMSID FROM SESSION.tmpTblCachedWalletIMSID WHERE ROWID < (@imsid_start_rowid + @imsid_per_batch)
				  ) T
			   	  INNER JOIN IMSSoci S ON (S.IMSID = T.IMSID AND S.sociID like '200%') 
				  INNER JOIN IMSSociAttribute SA1 ON (SA1.sociID = S.sociID AND SA1.ATTRNAME = 'keyCreationHost') 
				  INNER JOIN IMSSociAttribute SA2 ON (SA2.sociID = S.sociID AND SA2.ATTRNAME = 'keyCreationTime')
			  	GROUP BY
			  		S.IMSID, 
			  		SA1.ATTRVALUE
		  	)
			AND CREATIONDAYS > @wallet_retain_days
			AND CREATIONYYYYMMDD < @wallet_prune_before_yyyymmdd;
			
			IF (@debug_log_enabled = 1) THEN
				--SET @debug_log_event = '----SIMULATE_MODE_CALC_WALLET';	
				--Update SESSION.tmpTblCachedWalletLog SET LogTimeEnd = CURRENT TIMESTAMP WHERE LogID = @debug_logid AND LogEvent = @debug_log_event;
				Update SESSION.tmpTblCachedWalletLog SET LogTimeEnd = CURRENT TIMESTAMP WHERE LogID = @debug_logid AND LogSeq = @debug_logseq;
			END IF;
	  
		END IF;
		
		SET @info_total_wallet_deleted = @info_total_wallet_deleted + @wallet_del_per_batch;
		SET @info_total_imsid_processed = @info_total_imsid_processed + @imsid_per_batch;
		
		SET @total_unprocessed_imsid = @total_unprocessed_imsid - @imsid_per_batch;
		SET @info_total_wallet_processed = @info_total_wallet_processed + @wallet_per_batch;

		-- Delete current batch of IMS ID from temp table
		DELETE FROM SESSION.tmpTblCachedWalletImsID WHERE ROWID < (@imsid_start_rowid + @imsid_per_batch);
	
	END WHILE;
	-- ###[ END WHILE SESSION.tmpTblCachedWalletImsID IS NOT EMPTY) ]###--
	
	-- capture end time
	SELECT CURRENT TIMESTAMP INTO @info_end_timestamp FROM SYSIBM.SYSDUMMY1;
	
	IF (@debug_log_enabled = 1) THEN
		SET @debug_log_event = '[SCRIPT_END]';
		SET @debug_logseq = @debug_logseq +1;
		INSERT INTO SESSION.tmpTblCachedWalletLog (LogID, LogSeq, LogTimeStart, LogEvent) VALUES (@debug_logid, @debug_logseq, CURRENT TIMESTAMP, @debug_log_event);
	END IF;
		
	--calc elapsed time
	SELECT 	
		CASE WHEN DAY(time_diff)>0 THEN TRIM(CHAR(DAY(time_diff))) || 'd ' ELSE '' END
		|| CASE WHEN HOUR(time_diff)>0 THEN TRIM(CHAR(HOUR(time_diff))) || 'h ' ELSE '' END
		|| CASE WHEN MINUTE(time_diff)>0 THEN TRIM(CHAR(MINUTE(time_diff))) || 'm ' ELSE '' END
		|| TRIM(CHAR(SECOND(time_diff))) || 's ' 
		|| SUBSTR(TRIM(CHAR(MICROSECOND(CURRENT TIMESTAMP))),1,4) || 'ms' 
	INTO @info_elapsed_time 
		FROM ( SELECT (@info_end_timestamp - @info_start_timestamp) time_diff FROM SYSIBM.SYSDUMMY1);

	--cleanup all temporary tables if exist
	DROP TABLE SESSION.tmpTblCachedWalletDelete;
	DROP TABLE SESSION.tmpTblCachedWalletImsID;
	
	OPEN @cursor_result_output;
END
#
